package pathgame.tilemap;

import java.util.function.Function;
import pathgame.gameplay.PlayerAbilities;

/** Base class for tiles. Tiles are registered on construction.
 *
 * @author kajetan
 */
public class Tile
{
    private static int idCounter = 0;
    private static Tile[] tiles = new Tile[8];
    
    private static int addTile(Tile t)
    {
        if(idCounter >= tiles.length)
        {
            Tile[] newTiles = new Tile[tiles.length * 2];
            System.arraycopy(tiles, 0, newTiles, 0, tiles.length);
            tiles = newTiles;
        }
        tiles[idCounter] = t;
        return idCounter++;
    }
    
    /** Returns a tile from the given id or null if the id is invalid.
     *
     * @param id the id of the tile
     * @return a tile from the given id or null if the id is invalid
     */
    public static Tile fromId(int id)
    {
        if(id < 0 || id >= idCounter)
        {
            return null;
        }
        return tiles[id];
    }
    
    private final int id;
    private final int energyCost;
    private final float forestReplaceChance;
    private final Function<PlayerAbilities, Integer> speedUp;
    
    /** Creates a new tile, which is automatically registered.
     *
     */
    public Tile(int energyCost, float forestReplaceChance, Function<PlayerAbilities, Integer> speedUp)
    {
        id = addTile(this);
        this.energyCost = energyCost;
        this.forestReplaceChance = forestReplaceChance;
        this.speedUp = speedUp;
    }
    
    /** Creates a new tile, which is automatically registered.
     *
     */
    public Tile(int energyCost, Function<PlayerAbilities, Integer> speedUp)
    {
        this(energyCost, 1.0f, speedUp);
    }
    
    /** Creates a new tile, which is automatically registered.
     *
     */
    public Tile(int energyCost)
    {
        this(energyCost, 1.0f, (pa) -> 0);
    }

    /** Returns the id of the tile.
     *
     * @return the id of the tile
     */
    public final int getId()
    {
        return id;
    }
    
    public int getEnergyCost(PlayerAbilities pa)
    {
        return energyCost - speedUp.apply(pa);
    }

    /** Returns the chance that this tile is replaced by forest.
     *
     * @return the chance that this tile is replaced by forest
     */
    public float getForestReplaceChance()
    {
        return forestReplaceChance;
    } 
}
